//
//  SAScan.m
//
//
//  Created by 余西安 on 15/12/3.
//  Copyright © 2015年 Sian. All rights reserved.
//

#import "SAScan.h"

@implementation SAScan

- (instancetype)initWithFrame:(CGRect)frame
{
    if (self = [super initWithFrame:frame]){
        self.backgroundColor = [UIColor clearColor];
        UIImage *line = [UIImage imageNamed:@"scanLine.png"];
        self.scanLine = [[UIImageView alloc] initWithImage:line];
        [self.scanLine setHidden:YES];
        [self addSubview:self.scanLine];
    }
    return self;
}

- (void)drawRect:(CGRect)rect
{
    CGContextRef context = UIGraphicsGetCurrentContext();
    //非扫码区域半透明
    {
        CGFloat w = self.bounds.size.width * 0.7;
        CGFloat x = (self.bounds.size.width - w) * 0.5;
        CGFloat y = (self.bounds.size.height - w) * 0.5;
        {   // 外围区域填充半透明黑色
            // 设置非识别区域颜色
            CGContextSetRGBFillColor(context, 0, 0, 0, 0.6);
            
            // 扫码区域上面填充
            CGRect rect = CGRectMake(0, 0, self.frame.size.width, y);
            CGContextFillRect(context, rect);
            
            // 扫码区域左边填充
            rect = CGRectMake(0, y, x, w);
            CGContextFillRect(context, rect);
            
            // 扫码区域右边填充
            rect = CGRectMake(x + w, y, x, w);
            CGContextFillRect(context, rect);
            
            // 扫码区域下面填充
            rect = CGRectMake(0, y + w, self.frame.size.width, self.frame.size.height - y - w);
            CGContextFillRect(context, rect);
        }
        
        {   // 中间可视区域画边框
            UIColor *whiteColor = [UIColor colorWithWhite:1.0 alpha:0.5];
            CGContextSetStrokeColorWithColor(context, whiteColor.CGColor);
            CGContextSetLineWidth(context, 1);
            CGContextAddRect(context, CGRectMake(x, y, w, w));
            CGContextStrokePath(context);
        }
        
        
        {   // 中间可视区域画角框
            CGFloat lineWidth = 4.0f;
            CGFloat angleWidth = 15.0f;
            UIColor *greenColor = [UIColor colorWithRed:0 green:1.0 blue:0 alpha:0.8];
            CGContextSetLineWidth(context, lineWidth);
            CGContextSetStrokeColorWithColor(context, greenColor.CGColor);
            
            // 左上角
            CGContextMoveToPoint(context, x, y + angleWidth);
            CGContextAddLineToPoint(context, x, y);
            CGContextAddLineToPoint(context, x + angleWidth, y);
            // 右上角
            CGContextMoveToPoint(context, x + w - angleWidth, y);
            CGContextAddLineToPoint(context, x + w, y);
            CGContextAddLineToPoint(context, x + w, y + angleWidth);
            // 右下角
            CGContextMoveToPoint(context, x + w, y + w - angleWidth);
            CGContextAddLineToPoint(context, x + w, y + w);
            CGContextAddLineToPoint(context, x + w - angleWidth, y + w);
            // 左下角
            CGContextMoveToPoint(context, x + angleWidth, y + w);
            CGContextAddLineToPoint(context, x, y + w);
            CGContextAddLineToPoint(context, x, y + w - angleWidth);
        }
        CGContextStrokePath(context);
    }
}

- (void)layoutSubviews
{
    [super layoutSubviews];
    CGFloat width = self.bounds.size.width;
    CGFloat y = (self.bounds.size.height - width * 0.7) * 0.5;
    self.scanLine.frame = CGRectMake(0, y, width, 12);
}

- (void)startAnimation
{
    CGFloat width = self.bounds.size.width;
    CGFloat y = (self.bounds.size.height - width * 0.7) * 0.5;
    self.scanLine.frame = CGRectMake(0, y, width, 12);
    self.scanLine.hidden = NO;
    [UIView animateWithDuration:2.5 delay:0 options:UIViewAnimationOptionRepeat | UIViewAnimationOptionCurveLinear animations:^{
        self.scanLine.center = CGPointMake(width * 0.5, y + width * 0.7);
    } completion:^(BOOL finished) {
        if (finished) self.scanLine.center = CGPointMake(width * 0.5, y);
    }];
}

- (void)stopAnimation
{
    self.scanLine.hidden = YES;
    [self.scanLine.layer removeAllAnimations];
}
@end
