//
//  SAScanCtrl.m
//
//
//  Created by 余西安 on 15/12/3.
//  Copyright © 2015年 Sian. All rights reserved.
//

#import "SAScanCtrl.h"
#import "SAScan.h"
#import <AVFoundation/AVFoundation.h>

@interface SAScanCtrl ()<AVCaptureMetadataOutputObjectsDelegate>

@property (nonatomic, strong) AVCaptureSession *session;

@property (nonatomic, strong) AVAudioPlayer    *player;

@property (nonatomic, strong) SAScan           *scan;

@end

@implementation SAScanCtrl

- (instancetype)initWithBlock:(SAScanBlock)block
{
    if (self = [super init]){
        self.block = block;
    }
    return self;
}

- (void)viewDidLoad
{
    [super viewDidLoad];
    [self.navigationItem setTitle:@"扫一扫"];
    
    AVCaptureDevice *device = [AVCaptureDevice defaultDeviceWithMediaType:AVMediaTypeVideo];
    AVCaptureDeviceInput *input = [AVCaptureDeviceInput deviceInputWithDevice:device error:nil];
    AVCaptureMetadataOutput *output = [[AVCaptureMetadataOutput alloc]init];
    [output setMetadataObjectsDelegate:self queue:dispatch_get_main_queue()];   // 设置代理 在主线程里刷新
    [output setRectOfInterest:CGRectMake(0.12 , 0.12, 0.8, 0.8)];
    
    //初始化链接对象
    self.session = [[AVCaptureSession alloc] init];
    [self.session addInput:input];
    [self.session addOutput:output];
    [self.session setSessionPreset:AVCaptureSessionPresetHigh];
    //设置扫码支持的编码格式(如下设置条形码和二维码兼容)
    [output setMetadataObjectTypes:@[AVMetadataObjectTypeQRCode,AVMetadataObjectTypeEAN13Code, AVMetadataObjectTypeEAN8Code, AVMetadataObjectTypeCode128Code]];
    
    AVCaptureVideoPreviewLayer *layer = [AVCaptureVideoPreviewLayer layerWithSession:self.session];
    layer.videoGravity = AVLayerVideoGravityResizeAspectFill;
    layer.frame = self.view.bounds;

    [self.view.layer insertSublayer:layer atIndex:0];
    self.scan = [[SAScan alloc] initWithFrame:self.view.bounds];
    [self.view addSubview:self.scan];
    
    NSURL *url = [NSURL URLWithString:[[NSBundle mainBundle] pathForResource:@"scan" ofType:@"mp3"]];
    self.player = [[AVAudioPlayer alloc] initWithContentsOfURL:url error:nil];
    
    UIBarButtonItem *item = [[UIBarButtonItem alloc] initWithTitle:@"重试" style:UIBarButtonItemStylePlain target:self action:@selector(startRunning)];
    [self.navigationItem setRightBarButtonItem:item];
}

- (void)viewWillAppear:(BOOL)animated
{
    [super viewWillAppear:animated];
    [self.session startRunning];
}

- (void)viewDidAppear:(BOOL)animated
{
    [super viewDidAppear:animated];
    [self.scan startAnimation];
}

- (void)viewWillDisappear:(BOOL)animated
{
    [super viewWillDisappear:animated];
    [self.session stopRunning];
}

- (void)captureOutput:(AVCaptureOutput *)captureOutput didOutputMetadataObjects:(NSArray *)metadataObjects fromConnection:(AVCaptureConnection *)connection
{
    if (metadataObjects.count > 0){
        AVMetadataMachineReadableCodeObject *metadataObject = [metadataObjects firstObject];
        [self.player play];
        [self.scan stopAnimation];
        [self.session stopRunning];
        if (self.block) self.block(metadataObject.stringValue);
        [self.navigationController popViewControllerAnimated:YES];
        [self dismissViewControllerAnimated:YES completion:nil];
    }
}

- (void)startRunning
{
    [self.session startRunning];
    [self.scan startAnimation];
}
@end
